@@ -1,4 +1,3 @@
-use std::convert::TryFrom;
 use std::sync::Arc;
 
 use libc::{c_void, size_t, EINVAL, EIO};
@@ -11,8 +10,9 @@ use crate::io::{rustls_read_callback, CallbackReader, ReadCallback};
 use crate::rslice::{rustls_slice_bytes, rustls_str};
 use crate::server::rustls_server_config;
 use crate::{
-    ffi_panic_boundary, rustls_result, try_arc_from_ptr, try_callback, try_mut_from_ptr,
-    try_ref_from_ptr, BoxCastPtr, CastPtr,
+    ffi_panic_boundary, free_box, rustls_result, set_boxed_mut_ptr, to_box, to_boxed_mut_ptr,
+    try_callback, try_clone_arc, try_mut_from_ptr, try_ref_from_ptr, try_take, Castable,
+    OwnershipBox,
 };
 use rustls_result::NullParameter;
 
@@ -42,12 +42,11 @@ pub struct rustls_acceptor {
     _private: [u8; 0],
 }
 
-impl CastPtr for rustls_acceptor {
+impl Castable for rustls_acceptor {
+    type Ownership = OwnershipBox;
     type RustType = Acceptor;
 }
 
-impl BoxCastPtr for rustls_acceptor {}
-
 /// A parsed ClientHello produced by a rustls_acceptor. It is used to check
 /// server name indication (SNI), ALPN protocols, signature schemes, and
 /// cipher suites. It can be combined with a rustls_server_config to build a
@@ -56,12 +55,11 @@ pub struct rustls_accepted {
     _private: [u8; 0],
 }
 
-impl CastPtr for rustls_accepted {
+impl Castable for rustls_accepted {
+    type Ownership = OwnershipBox;
     type RustType = Option<Accepted>;
 }
 
-impl BoxCastPtr for rustls_accepted {}
-
 impl rustls_acceptor {
     /// Create and return a new rustls_acceptor.
     ///
@@ -70,7 +68,7 @@ impl rustls_acceptor {
     #[no_mangle]
     pub extern "C" fn rustls_acceptor_new() -> *mut rustls_acceptor {
         ffi_panic_boundary! {
-            BoxCastPtr::to_mut_ptr(Acceptor::default())
+            to_boxed_mut_ptr(Acceptor::default())
         }
     }
 
@@ -84,7 +82,7 @@ impl rustls_acceptor {
     #[no_mangle]
     pub extern "C" fn rustls_acceptor_free(acceptor: *mut rustls_acceptor) {
         ffi_panic_boundary! {
-            BoxCastPtr::to_box(acceptor);
+            to_box(acceptor);
         }
     }
 
@@ -179,13 +177,13 @@ impl rustls_acceptor {
         ffi_panic_boundary! {
             let acceptor: &mut Acceptor = try_mut_from_ptr!(acceptor);
             if out_accepted.is_null() {
-                return NullParameter
+                return NullParameter;
             }
             match acceptor.accept() {
                 Ok(None) => rustls_result::AcceptorNotReady,
                 Err(e) => map_error(e),
                 Ok(Some(accepted)) => {
-                    BoxCastPtr::set_mut_ptr(out_accepted, Some(accepted));
+                    set_boxed_mut_ptr(out_accepted, Some(accepted));
                     rustls_result::Ok
                 }
             }
@@ -400,17 +398,14 @@ impl rustls_accepted {
     ) -> rustls_result {
         ffi_panic_boundary! {
             let accepted: &mut Option<Accepted> = try_mut_from_ptr!(accepted);
-            let accepted = match accepted.take() {
-                Some(a) => a,
-                None => return rustls_result::AlreadyUsed,
-            };
-            let config: Arc<ServerConfig> = try_arc_from_ptr!(config);
+            let accepted = try_take!(accepted);
+            let config: Arc<ServerConfig> = try_clone_arc!(config);
             match accepted.into_connection(config) {
                 Ok(built) => {
                     let wrapped = crate::connection::Connection::from_server(built);
-                    BoxCastPtr::set_mut_ptr(out_conn, wrapped);
+                    set_boxed_mut_ptr(out_conn, wrapped);
                     rustls_result::Ok
-                },
+                }
                 Err(e) => map_error(e),
             }
         }
@@ -426,7 +421,7 @@ impl rustls_accepted {
     #[no_mangle]
     pub extern "C" fn rustls_accepted_free(accepted: *mut rustls_accepted) {
         ffi_panic_boundary! {
-            BoxCastPtr::to_box(accepted);
+            free_box(accepted);
         }
     }
 }
@@ -442,7 +437,6 @@ mod tests {
 
     use crate::cipher::rustls_certified_key;
     use crate::client::{rustls_client_config, rustls_client_config_builder};
-    use crate::connection::rustls_connection;
     use crate::server::rustls_server_config_builder;
 
     use super::*;
