@@ -3,9 +3,9 @@ use std::{ffi::c_void, ptr::null};
 use std::{ptr::null_mut, slice};
 
 use libc::{size_t, EINVAL, EIO};
-use rustls::{
-    Certificate, ClientConnection, ServerConnection, SupportedCipherSuite, ALL_CIPHER_SUITES,
-};
+use pki_types::CertificateDer;
+use rustls::crypto::ring::ALL_CIPHER_SUITES;
+use rustls::{ClientConnection, ServerConnection, SupportedCipherSuite};
 
 use crate::io::{
     rustls_write_vectored_callback, CallbackReader, CallbackWriter, ReadCallback,
@@ -13,15 +13,15 @@ use crate::io::{
 };
 use crate::log::{ensure_log_registered, rustls_log_callback};
 
-use crate::BoxCastPtr;
 use crate::{
     cipher::{rustls_certificate, rustls_supported_ciphersuite},
     error::{map_error, rustls_io_result, rustls_result},
+    ffi_panic_boundary, free_box,
     io::{rustls_read_callback, rustls_write_callback},
-    try_callback,
+    try_callback, try_mut_from_ptr, try_ref_from_ptr, try_slice, userdata_push, Castable,
+    OwnershipBox,
 };
-use crate::{ffi_panic_boundary, try_ref_from_ptr};
-use crate::{try_mut_from_ptr, try_slice, userdata_push, CastPtr};
+
 use rustls_result::NullParameter;
 
 pub(crate) struct Connection {
@@ -97,12 +97,11 @@ pub struct rustls_connection {
     _private: [u8; 0],
 }
 
-impl CastPtr for rustls_connection {
+impl Castable for rustls_connection {
+    type Ownership = OwnershipBox;
     type RustType = Connection;
 }
 
-impl BoxCastPtr for rustls_connection {}
-
 impl rustls_connection {
     /// Set the userdata pointer associated with this connection. This will be passed
     /// to any callbacks invoked by the connection, if you've set up callbacks in the config.
@@ -149,7 +148,7 @@ impl rustls_connection {
         ffi_panic_boundary! {
             let conn: &mut Connection = try_mut_from_ptr!(conn);
             if out_n.is_null() {
-                return rustls_io_result(EINVAL)
+                return rustls_io_result(EINVAL);
             }
             let callback: ReadCallback = try_callback!(callback);
 
@@ -186,7 +185,7 @@ impl rustls_connection {
         ffi_panic_boundary! {
             let conn: &mut Connection = try_mut_from_ptr!(conn);
             if out_n.is_null() {
-                return rustls_io_result(EINVAL)
+                return rustls_io_result(EINVAL);
             }
             let callback: WriteCallback = try_callback!(callback);
 
@@ -223,7 +222,7 @@ impl rustls_connection {
         ffi_panic_boundary! {
             let conn: &mut Connection = try_mut_from_ptr!(conn);
             if out_n.is_null() {
-                return rustls_io_result(EINVAL)
+                return rustls_io_result(EINVAL);
             }
             let callback: VectoredWriteCallback = try_callback!(callback);
 
@@ -325,15 +324,15 @@ impl rustls_connection {
     ///  `const struct rustls_connection *`).
     /// <https://docs.rs/rustls/latest/rustls/enum.Connection.html#method.peer_certificates>
     #[no_mangle]
-    pub extern "C" fn rustls_connection_get_peer_certificate(
+    pub extern "C" fn rustls_connection_get_peer_certificate<'a>(
         conn: *const rustls_connection,
         i: size_t,
-    ) -> *const rustls_certificate {
+    ) -> *const rustls_certificate<'a> {
         ffi_panic_boundary! {
             let conn: &Connection = try_ref_from_ptr!(conn);
             match conn.peer_certificates().and_then(|c| c.get(i)) {
-                Some(cert) => cert as *const Certificate as *const _,
-                None => null()
+                Some(cert) => cert as *const CertificateDer as *const _,
+                None => null(),
             }
         }
     }
@@ -358,7 +357,7 @@ impl rustls_connection {
         ffi_panic_boundary! {
             let conn: &Connection = try_ref_from_ptr!(conn);
             if protocol_out.is_null() || protocol_out_len.is_null() {
-                return
+                return;
             }
             match conn.alpn_protocol() {
                 Some(p) => unsafe {
@@ -368,7 +367,7 @@ impl rustls_connection {
                 None => unsafe {
                     *protocol_out = null();
                     *protocol_out_len = 0;
-                }
+                },
             }
         }
     }
@@ -435,7 +434,7 @@ impl rustls_connection {
             let conn: &mut Connection = try_mut_from_ptr!(conn);
             let write_buf: &[u8] = try_slice!(buf, count);
             if out_n.is_null() {
-                return NullParameter
+                return NullParameter;
             }
             let n_written: usize = match conn.writer().write(write_buf) {
                 Ok(n) => n,
@@ -470,22 +469,24 @@ impl rustls_connection {
         ffi_panic_boundary! {
             let conn: &mut Connection = try_mut_from_ptr!(conn);
             if buf.is_null() {
-                return NullParameter
+                return NullParameter;
             }
             if out_n.is_null() {
-                return NullParameter
+                return NullParameter;
             }
 
             // Safety: the memory pointed at by buf must be initialized
             // (required by documentation of this function).
-            let read_buf: &mut [u8] = unsafe {
-                slice::from_raw_parts_mut(buf, count)
-            };
+            let read_buf: &mut [u8] = unsafe { slice::from_raw_parts_mut(buf, count) };
 
             let n_read: usize = match conn.reader().read(read_buf) {
                 Ok(n) => n,
-                Err(e) if e.kind() == ErrorKind::UnexpectedEof => return rustls_result::UnexpectedEof,
-                Err(e) if e.kind() == ErrorKind::WouldBlock => return rustls_result::PlaintextEmpty,
+                Err(e) if e.kind() == ErrorKind::UnexpectedEof => {
+                    return rustls_result::UnexpectedEof
+                }
+                Err(e) if e.kind() == ErrorKind::WouldBlock => {
+                    return rustls_result::PlaintextEmpty
+                }
                 Err(_) => return rustls_result::Io,
             };
             unsafe {
@@ -519,18 +520,21 @@ impl rustls_connection {
         ffi_panic_boundary! {
             let conn: &mut Connection = try_mut_from_ptr!(conn);
             if buf.is_null() || out_n.is_null() {
-                return NullParameter
+                return NullParameter;
             }
-            let read_buf: &mut [std::mem::MaybeUninit<u8>] = unsafe {
-                slice::from_raw_parts_mut(buf, count)
-            };
+            let read_buf: &mut [std::mem::MaybeUninit<u8>] =
+                unsafe { slice::from_raw_parts_mut(buf, count) };
 
             let mut read_buf: std::io::BorrowedBuf<'_> = read_buf.into();
 
             let n_read: usize = match conn.reader().read_buf(read_buf.unfilled()) {
                 Ok(()) => read_buf.filled().len(),
-                Err(e) if e.kind() == ErrorKind::UnexpectedEof => return rustls_result::UnexpectedEof,
-                Err(e) if e.kind() == ErrorKind::WouldBlock => return rustls_result::PlaintextEmpty,
+                Err(e) if e.kind() == ErrorKind::UnexpectedEof => {
+                    return rustls_result::UnexpectedEof
+                }
+                Err(e) if e.kind() == ErrorKind::WouldBlock => {
+                    return rustls_result::PlaintextEmpty
+                }
                 Err(_) => return rustls_result::Io,
             };
             unsafe {
@@ -545,9 +549,7 @@ impl rustls_connection {
     #[no_mangle]
     pub extern "C" fn rustls_connection_free(conn: *mut rustls_connection) {
         ffi_panic_boundary! {
-            let conn: &mut Connection = try_mut_from_ptr!(conn);
-            // Convert the pointer to a Box and drop it.
-            drop(unsafe { Box::from_raw(conn) });
+            free_box(conn);
         }
     }
 }
