@@ -1,12 +1,11 @@
 use std::cmp::min;
-use std::convert::TryFrom;
 use std::fmt::Display;
 use std::sync::Arc;
 
 use crate::ffi_panic_boundary;
 use libc::{c_char, c_uint, size_t};
-use num_enum::TryFromPrimitive;
-use rustls::{CertificateError, Error, InvalidMessage};
+use rustls::server::VerifierBuilderError;
+use rustls::{CertRevocationListError, CertificateError, Error, InvalidMessage};
 
 /// A return value for a function that may return either success (0) or a
 /// non-zero value representing an error. The values should match socket
@@ -16,6 +15,180 @@ use rustls::{CertificateError, Error, InvalidMessage};
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
 pub struct rustls_io_result(pub libc::c_int);
 
+macro_rules! u32_enum_builder {
+    (
+    $(#[$comment:meta])*
+        EnumName: $enum_name: ident;
+        EnumDefault: $enum_default: ident;
+        EnumVal { $( $enum_var: ident => $enum_val: expr ),* }
+    ) => {
+        $(#[$comment])*
+        #[allow(dead_code)]
+        #[repr(u32)]
+        #[derive(Debug, Clone, Copy, PartialEq, Eq)]
+        pub enum $enum_name {
+            $( $enum_var = $enum_val),*
+        }
+        impl From<u32> for $enum_name {
+            fn from(x: u32) -> Self {
+                match x {
+                    $($enum_val => $enum_name::$enum_var),*
+                    , _ => $enum_name::$enum_default,
+                }
+            }
+        }
+    };
+}
+
+u32_enum_builder! {
+    EnumName: rustls_result;
+    EnumDefault: InvalidParameter;
+    EnumVal{
+        Ok => 7000,
+        Io => 7001,
+        NullParameter => 7002,
+        InvalidDnsNameError => 7003,
+        Panic => 7004,
+        CertificateParseError => 7005,
+        PrivateKeyParseError => 7006,
+        InsufficientSize => 7007,
+        NotFound => 7008,
+        InvalidParameter => 7009,
+        UnexpectedEof => 7010,
+        PlaintextEmpty => 7011,
+        AcceptorNotReady => 7012,
+        AlreadyUsed => 7013,
+        CertificateRevocationListParseError => 7014,
+
+        // From https://docs.rs/rustls/latest/rustls/enum.Error.html
+        NoCertificatesPresented => 7101,
+        DecryptError => 7102,
+        FailedToGetCurrentTime => 7103,
+        FailedToGetRandomBytes => 7113,
+        HandshakeNotComplete => 7104,
+        PeerSentOversizedRecord => 7105,
+        NoApplicationProtocol => 7106,
+        BadMaxFragmentSize => 7114,
+        UnsupportedNameType => 7115,
+        EncryptError => 7116,
+
+        // Reserved from previous use pre rustls-ffi <0.21.0
+        //  CorruptMessage => 7100,
+        //  CorruptMessagePayload => 7111,
+        //  CertInvalidEncoding => 7117,
+        //  CertInvalidSignatureType => 7118,
+        //  CertInvalidSignature => 7119,
+        //  CertInvalidData => 7120,
+
+        // From InvalidCertificate, with fields that get flattened.
+        // https://docs.rs/rustls/0.21.0/rustls/enum.Error.html#variant.InvalidCertificate
+        CertEncodingBad => 7121,
+        CertExpired => 7122,
+        CertNotYetValid => 7123,
+        CertRevoked => 7124,
+        CertUnhandledCriticalExtension => 7125,
+        CertUnknownIssuer => 7126,
+        CertBadSignature => 7127,
+        CertNotValidForName => 7128,
+        CertInvalidPurpose => 7129,
+        CertApplicationVerificationFailure => 7130,
+        CertOtherError => 7131,
+
+        // From InvalidMessage, with fields that get flattened.
+        // https://docs.rs/rustls/0.21.0/rustls/enum.Error.html#variant.InvalidMessage
+        MessageHandshakePayloadTooLarge => 7133,
+        MessageInvalidCcs => 7134,
+        MessageInvalidContentType => 7135,
+        MessageInvalidCertStatusType => 7136,
+        MessageInvalidCertRequest => 7137,
+        MessageInvalidDhParams => 7138,
+        MessageInvalidEmptyPayload => 7139,
+        MessageInvalidKeyUpdate => 7140,
+        MessageInvalidServerName => 7141,
+        MessageTooLarge => 7142,
+        MessageTooShort => 7143,
+        MessageMissingData => 7144,
+        MessageMissingKeyExchange => 7145,
+        MessageNoSignatureSchemes => 7146,
+        MessageTrailingData => 7147,
+        MessageUnexpectedMessage => 7148,
+        MessageUnknownProtocolVersion => 7149,
+        MessageUnsupportedCompression => 7150,
+        MessageUnsupportedCurveType => 7151,
+        MessageUnsupportedKeyExchangeAlgorithm => 7152,
+        MessageInvalidOther => 7153, // Last added.
+
+        // From Error, with fields that get dropped.
+        PeerIncompatibleError => 7107,
+        PeerMisbehavedError => 7108,
+        InappropriateMessage => 7109,
+        InappropriateHandshakeMessage => 7110,
+        General => 7112,
+
+        // From Error, with fields that get flattened.
+        // https://docs.rs/rustls/latest/rustls/internal/msgs/enums/enum.AlertDescription.html
+        AlertCloseNotify => 7200,
+        AlertUnexpectedMessage => 7201,
+        AlertBadRecordMac => 7202,
+        AlertDecryptionFailed => 7203,
+        AlertRecordOverflow => 7204,
+        AlertDecompressionFailure => 7205,
+        AlertHandshakeFailure => 7206,
+        AlertNoCertificate => 7207,
+        AlertBadCertificate => 7208,
+        AlertUnsupportedCertificate => 7209,
+        AlertCertificateRevoked => 7210,
+        AlertCertificateExpired => 7211,
+        AlertCertificateUnknown => 7212,
+        AlertIllegalParameter => 7213,
+        AlertUnknownCA => 7214,
+        AlertAccessDenied => 7215,
+        AlertDecodeError => 7216,
+        AlertDecryptError => 7217,
+        AlertExportRestriction => 7218,
+        AlertProtocolVersion => 7219,
+        AlertInsufficientSecurity => 7220,
+        AlertInternalError => 7221,
+        AlertInappropriateFallback => 7222,
+        AlertUserCanceled => 7223,
+        AlertNoRenegotiation => 7224,
+        AlertMissingExtension => 7225,
+        AlertUnsupportedExtension => 7226,
+        AlertCertificateUnobtainable => 7227,
+        AlertUnrecognisedName => 7228,
+        AlertBadCertificateStatusResponse => 7229,
+        AlertBadCertificateHashValue => 7230,
+        AlertUnknownPSKIdentity => 7231,
+        AlertCertificateRequired => 7232,
+        AlertNoApplicationProtocol => 7233,
+        AlertUnknown => 7234,
+
+        // Reserved from previous use pre rustls-ffi <0.22.0
+        // CertSCTMalformed => 7319,
+        // CertSCTInvalidSignature => 7320,
+        // CertSCTTimestampInFuture => 7321,
+        // CertSCTUnsupportedVersion => 7322,
+        // CertSCTUnknownLog => 7323,
+
+        // From InvalidCertRevocationList, with fields that get flattened.
+        // https://docs.rs/rustls/0.21.6/rustls/enum.Error.html#variant.InvalidCertRevocationList
+        CertRevocationListBadSignature => 7400,
+        CertRevocationListInvalidCrlNumber => 7401,
+        CertRevocationListInvalidRevokedCertSerialNumber => 7402,
+        CertRevocationListIssuerInvalidForCrl => 7403,
+        CertRevocationListOtherError => 7404,
+        CertRevocationListParseError => 7405,
+        CertRevocationListUnsupportedCrlVersion => 7406,
+        CertRevocationListUnsupportedCriticalExtension => 7407,
+        CertRevocationListUnsupportedDeltaCrl => 7408,
+        CertRevocationListUnsupportedIndirectCrl => 7409,
+        CertRevocationListUnsupportedRevocationReason => 7410,
+
+        // From ClientCertVerifierBuilderError, with fields that get flattened.
+        ClientCertVerifierBuilderNoRootAnchors => 7500
+    }
+}
+
 impl rustls_result {
     /// After a rustls function returns an error, you may call
     /// this to get a pointer to a buffer containing a detailed error
@@ -30,13 +203,12 @@ impl rustls_result {
     ) {
         ffi_panic_boundary! {
             if buf.is_null() {
-                return
+                return;
             }
             if out_n.is_null() {
-                return
+                return;
             }
-            let result: rustls_result = rustls_result::try_from(result).unwrap_or(rustls_result::InvalidParameter);
-            let error_str = result.to_string();
+            let error_str = rustls_result::from(result).to_string();
             let out_len: usize = min(len - 1, error_str.len());
             unsafe {
                 std::ptr::copy_nonoverlapping(error_str.as_ptr() as *mut c_char, buf, out_len);
@@ -47,11 +219,9 @@ impl rustls_result {
 
     #[no_mangle]
     pub extern "C" fn rustls_result_is_cert_error(result: c_uint) -> bool {
-        let result: rustls_result =
-            rustls_result::try_from(result).unwrap_or(rustls_result::InvalidParameter);
         use rustls_result::*;
         matches!(
-            result,
+            rustls_result::from(result),
             CertEncodingBad
                 | CertExpired
                 | CertNotYetValid
@@ -63,11 +233,6 @@ impl rustls_result {
                 | CertInvalidPurpose
                 | CertApplicationVerificationFailure
                 | CertOtherError
-                | CertSCTMalformed
-                | CertSCTInvalidSignature
-                | CertSCTTimestampInFuture
-                | CertSCTUnsupportedVersion
-                | CertSCTUnknownLog
         )
     }
 }
@@ -76,6 +241,7 @@ impl rustls_result {
 /// inputs, including Ok, return rustls::Error::General.
 pub(crate) fn cert_result_to_error(result: rustls_result) -> rustls::Error {
     use rustls::Error::*;
+    use rustls::OtherError;
     use rustls_result::*;
     match result {
         CertEncodingBad => InvalidCertificate(CertificateError::BadEncoding),
@@ -92,12 +258,9 @@ pub(crate) fn cert_result_to_error(result: rustls_result) -> rustls::Error {
         CertApplicationVerificationFailure => {
             InvalidCertificate(CertificateError::ApplicationVerificationFailure)
         }
-        CertOtherError => InvalidCertificate(CertificateError::Other(Arc::from(Box::from("")))),
-        CertSCTMalformed => InvalidSct(sct::Error::MalformedSct),
-        CertSCTInvalidSignature => InvalidSct(sct::Error::InvalidSignature),
-        CertSCTTimestampInFuture => InvalidSct(sct::Error::TimestampInFuture),
-        CertSCTUnsupportedVersion => InvalidSct(sct::Error::UnsupportedSctVersion),
-        CertSCTUnknownLog => InvalidSct(sct::Error::UnknownLog),
+        CertOtherError => InvalidCertificate(CertificateError::Other(OtherError(Arc::from(
+            Box::from(""),
+        )))),
         _ => rustls::Error::General("".into()),
     }
 }
@@ -128,147 +291,11 @@ fn test_rustls_result_is_cert_error() {
     for id in 7121..=7131 {
         assert!(rustls_result::rustls_result_is_cert_error(id));
     }
-
-    // Test SCTError range
-    for id in 7319..=7323 {
-        assert!(rustls_result::rustls_result_is_cert_error(id));
-    }
-}
-
-#[allow(dead_code)]
-#[repr(u32)]
-#[derive(Debug, TryFromPrimitive, Clone, Copy, PartialEq, Eq)]
-pub enum rustls_result {
-    Ok = 7000,
-    Io = 7001,
-    NullParameter = 7002,
-    InvalidDnsNameError = 7003,
-    Panic = 7004,
-    CertificateParseError = 7005,
-    PrivateKeyParseError = 7006,
-    InsufficientSize = 7007,
-    NotFound = 7008,
-    InvalidParameter = 7009,
-    UnexpectedEof = 7010,
-    PlaintextEmpty = 7011,
-    AcceptorNotReady = 7012,
-    AlreadyUsed = 7013,
-
-    // From https://docs.rs/rustls/latest/rustls/enum.Error.html
-    NoCertificatesPresented = 7101,
-    DecryptError = 7102,
-    FailedToGetCurrentTime = 7103,
-    FailedToGetRandomBytes = 7113,
-    HandshakeNotComplete = 7104,
-    PeerSentOversizedRecord = 7105,
-    NoApplicationProtocol = 7106,
-    BadMaxFragmentSize = 7114,
-    UnsupportedNameType = 7115,
-    EncryptError = 7116,
-
-    // Reserved from previous use pre rustls-ffi <0.21.0
-    //  CorruptMessage = 7100,
-    //  CorruptMessagePayload = 7111,
-    //  CertInvalidEncoding = 7117,
-    //  CertInvalidSignatureType = 7118,
-    //  CertInvalidSignature = 7119,
-    //  CertInvalidData = 7120,
-
-    // From InvalidCertificate, with fields that get flattened.
-    // https://docs.rs/rustls/0.21.0/rustls/enum.Error.html#variant.InvalidCertificate
-    CertEncodingBad = 7121,
-    CertExpired = 7122,
-    CertNotYetValid = 7123,
-    CertRevoked = 7124,
-    CertUnhandledCriticalExtension = 7125,
-    CertUnknownIssuer = 7126,
-    CertBadSignature = 7127,
-    CertNotValidForName = 7128,
-    CertInvalidPurpose = 7129,
-    CertApplicationVerificationFailure = 7130,
-    CertOtherError = 7131,
-
-    // From InvalidMessage, with fields that get flattened.
-    // https://docs.rs/rustls/0.21.0/rustls/enum.Error.html#variant.InvalidMessage
-    MessageHandshakePayloadTooLarge = 7133,
-    MessageInvalidCcs = 7134,
-    MessageInvalidContentType = 7135,
-    MessageInvalidCertStatusType = 7136,
-    MessageInvalidCertRequest = 7137,
-    MessageInvalidDhParams = 7138,
-    MessageInvalidEmptyPayload = 7139,
-    MessageInvalidKeyUpdate = 7140,
-    MessageInvalidServerName = 7141,
-    MessageTooLarge = 7142,
-    MessageTooShort = 7143,
-    MessageMissingData = 7144,
-    MessageMissingKeyExchange = 7145,
-    MessageNoSignatureSchemes = 7146,
-    MessageTrailingData = 7147,
-    MessageUnexpectedMessage = 7148,
-    MessageUnknownProtocolVersion = 7149,
-    MessageUnsupportedCompression = 7150,
-    MessageUnsupportedCurveType = 7151,
-    MessageUnsupportedKeyExchangeAlgorithm = 7152,
-    MessageInvalidOther = 7153, // Last added.
-
-    // From Error, with fields that get dropped.
-    PeerIncompatibleError = 7107,
-    PeerMisbehavedError = 7108,
-    InappropriateMessage = 7109,
-    InappropriateHandshakeMessage = 7110,
-    General = 7112,
-
-    // From Error, with fields that get flattened.
-    // https://docs.rs/rustls/latest/rustls/internal/msgs/enums/enum.AlertDescription.html
-    AlertCloseNotify = 7200,
-    AlertUnexpectedMessage = 7201,
-    AlertBadRecordMac = 7202,
-    AlertDecryptionFailed = 7203,
-    AlertRecordOverflow = 7204,
-    AlertDecompressionFailure = 7205,
-    AlertHandshakeFailure = 7206,
-    AlertNoCertificate = 7207,
-    AlertBadCertificate = 7208,
-    AlertUnsupportedCertificate = 7209,
-    AlertCertificateRevoked = 7210,
-    AlertCertificateExpired = 7211,
-    AlertCertificateUnknown = 7212,
-    AlertIllegalParameter = 7213,
-    AlertUnknownCA = 7214,
-    AlertAccessDenied = 7215,
-    AlertDecodeError = 7216,
-    AlertDecryptError = 7217,
-    AlertExportRestriction = 7218,
-    AlertProtocolVersion = 7219,
-    AlertInsufficientSecurity = 7220,
-    AlertInternalError = 7221,
-    AlertInappropriateFallback = 7222,
-    AlertUserCanceled = 7223,
-    AlertNoRenegotiation = 7224,
-    AlertMissingExtension = 7225,
-    AlertUnsupportedExtension = 7226,
-    AlertCertificateUnobtainable = 7227,
-    AlertUnrecognisedName = 7228,
-    AlertBadCertificateStatusResponse = 7229,
-    AlertBadCertificateHashValue = 7230,
-    AlertUnknownPSKIdentity = 7231,
-    AlertCertificateRequired = 7232,
-    AlertNoApplicationProtocol = 7233,
-    AlertUnknown = 7234,
-
-    // https://docs.rs/sct/latest/sct/enum.Error.html
-    CertSCTMalformed = 7319,
-    CertSCTInvalidSignature = 7320,
-    CertSCTTimestampInFuture = 7321,
-    CertSCTUnsupportedVersion = 7322,
-    CertSCTUnknownLog = 7323,
 }
 
 pub(crate) fn map_error(input: rustls::Error) -> rustls_result {
     use rustls::AlertDescription as alert;
     use rustls_result::*;
-    use sct::Error as sct;
 
     match input {
         Error::InappropriateMessage { .. } => InappropriateMessage,
@@ -366,22 +393,52 @@ pub(crate) fn map_error(input: rustls::Error) -> rustls_result {
             alert::Unknown(_) => AlertUnknown,
             _ => AlertUnknown,
         },
-        Error::InvalidSct(e) => match e {
-            sct::MalformedSct => CertSCTMalformed,
-            sct::InvalidSignature => CertSCTInvalidSignature,
-            sct::TimestampInFuture => CertSCTTimestampInFuture,
-            sct::UnsupportedSctVersion => CertSCTUnsupportedVersion,
-            sct::UnknownLog => CertSCTUnknownLog,
-        },
+
+        Error::InvalidCertRevocationList(e) => map_crl_error(e),
+
         _ => General,
     }
 }
 
+pub(crate) fn map_crl_error(err: CertRevocationListError) -> rustls_result {
+    use rustls_result::*;
+
+    match err {
+        CertRevocationListError::BadSignature => CertRevocationListBadSignature,
+        CertRevocationListError::InvalidCrlNumber => CertRevocationListInvalidCrlNumber,
+        CertRevocationListError::InvalidRevokedCertSerialNumber => {
+            CertRevocationListInvalidRevokedCertSerialNumber
+        }
+        CertRevocationListError::IssuerInvalidForCrl => CertRevocationListIssuerInvalidForCrl,
+        CertRevocationListError::Other(_) => CertRevocationListOtherError,
+        CertRevocationListError::ParseError => CertRevocationListParseError,
+        CertRevocationListError::UnsupportedCrlVersion => CertRevocationListUnsupportedCrlVersion,
+        CertRevocationListError::UnsupportedCriticalExtension => {
+            CertRevocationListUnsupportedCriticalExtension
+        }
+        CertRevocationListError::UnsupportedDeltaCrl => CertRevocationListUnsupportedDeltaCrl,
+        CertRevocationListError::UnsupportedIndirectCrl => CertRevocationListUnsupportedIndirectCrl,
+        CertRevocationListError::UnsupportedRevocationReason => {
+            CertRevocationListUnsupportedRevocationReason
+        }
+        _ => CertRevocationListOtherError,
+    }
+}
+
+pub(crate) fn map_verifier_builder_error(err: VerifierBuilderError) -> rustls_result {
+    match err {
+        VerifierBuilderError::NoRootAnchors => {
+            rustls_result::ClientCertVerifierBuilderNoRootAnchors
+        }
+        VerifierBuilderError::InvalidCrl(crl_err) => map_crl_error(crl_err),
+        _ => rustls_result::General,
+    }
+}
+
 impl Display for rustls_result {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         use rustls::AlertDescription as alert;
         use rustls_result::*;
-        use sct::Error as sct;
 
         match self {
             // These variants are local to this glue layer.
@@ -414,6 +471,9 @@ impl Display for rustls_result {
                 f,
                 "tried to use a rustls struct after it had been converted to another struct"
             ),
+            CertificateRevocationListParseError => {
+                write!(f, "error parsing certificate revocation list (CRL)",)
+            }
 
             CertEncodingBad => Error::InvalidCertificate(CertificateError::BadEncoding).fmt(f),
             CertExpired => Error::InvalidCertificate(CertificateError::Expired).fmt(f),
@@ -558,11 +618,48 @@ impl Display for rustls_result {
             AlertNoApplicationProtocol => Error::AlertReceived(alert::NoApplicationProtocol).fmt(f),
             AlertUnknown => Error::AlertReceived(alert::Unknown(0)).fmt(f),
 
-            CertSCTMalformed => Error::InvalidSct(sct::MalformedSct).fmt(f),
-            CertSCTInvalidSignature => Error::InvalidSct(sct::InvalidSignature).fmt(f),
-            CertSCTTimestampInFuture => Error::InvalidSct(sct::TimestampInFuture).fmt(f),
-            CertSCTUnsupportedVersion => Error::InvalidSct(sct::UnsupportedSctVersion).fmt(f),
-            CertSCTUnknownLog => Error::InvalidSct(sct::UnknownLog).fmt(f),
+            CertRevocationListBadSignature => {
+                Error::InvalidCertRevocationList(CertRevocationListError::BadSignature).fmt(f)
+            }
+            CertRevocationListInvalidCrlNumber => {
+                Error::InvalidCertRevocationList(CertRevocationListError::InvalidCrlNumber).fmt(f)
+            }
+            CertRevocationListInvalidRevokedCertSerialNumber => Error::InvalidCertRevocationList(
+                CertRevocationListError::InvalidRevokedCertSerialNumber,
+            )
+            .fmt(f),
+            CertRevocationListIssuerInvalidForCrl => {
+                Error::InvalidCertRevocationList(CertRevocationListError::IssuerInvalidForCrl)
+                    .fmt(f)
+            }
+            CertRevocationListOtherError => {
+                write!(f, "unknown certificate revocation list (CRL) error")
+            }
+            CertRevocationListParseError => {
+                Error::InvalidCertRevocationList(CertRevocationListError::ParseError).fmt(f)
+            }
+            CertRevocationListUnsupportedCrlVersion => {
+                Error::InvalidCertRevocationList(CertRevocationListError::UnsupportedCrlVersion)
+                    .fmt(f)
+            }
+            CertRevocationListUnsupportedCriticalExtension => Error::InvalidCertRevocationList(
+                CertRevocationListError::UnsupportedCriticalExtension,
+            )
+            .fmt(f),
+            CertRevocationListUnsupportedDeltaCrl => {
+                Error::InvalidCertRevocationList(CertRevocationListError::UnsupportedDeltaCrl)
+                    .fmt(f)
+            }
+            CertRevocationListUnsupportedIndirectCrl => {
+                Error::InvalidCertRevocationList(CertRevocationListError::UnsupportedIndirectCrl)
+                    .fmt(f)
+            }
+            CertRevocationListUnsupportedRevocationReason => Error::InvalidCertRevocationList(
+                CertRevocationListError::UnsupportedRevocationReason,
+            )
+            .fmt(f),
+
+            ClientCertVerifierBuilderNoRootAnchors => write!(f, "no root trust anchors provided"),
         }
     }
 }
