@@ -1,7 +1,10 @@
 #![crate_type = "staticlib"]
 #![allow(non_camel_case_types)]
 #![allow(clippy::not_unsafe_ptr_arg_deref)]
+// TODO(#333): Fix this clippy warning.
+#![allow(clippy::arc_with_non_send_sync)]
 #![cfg_attr(feature = "read_buf", feature(read_buf))]
+#![cfg_attr(feature = "read_buf", feature(core_io_borrowed_buf))]
 
 //! This package contains bindings for using rustls via a C API. If
 //! you're looking at this on docs.rs, [you may want the rustls docs
@@ -51,6 +54,7 @@ include!(concat!(env!("OUT_DIR"), "/version.rs"));
 // Rust code, we model these thread locals as a stack, so we can always
 // restore the previous version.
 thread_local! {
+    #[allow(clippy::thread_local_initializer_can_be_made_const)]
     pub(crate) static USERDATA: RefCell<Vec<Userdata>> = RefCell::new(Vec::new());
 }
 
@@ -62,7 +66,7 @@ pub(crate) struct Userdata {
 /// UserdataGuard pops an entry off the USERDATA stack, restoring the
 /// thread-local state to its value previous to the creation of the UserdataGuard.
 /// Invariants: As long as a UserdataGuard is live:
-//// - The stack of userdata items for this thread must have at least one item.
+///  - The stack of userdata items for this thread must have at least one item.
 ///  - The top item on that stack must be the one this guard was built with.
 ///  - The `data` field must not be None.
 /// If any of these invariants fails, try_drop will return an error.
@@ -290,115 +294,323 @@ mod tests {
     }
 }
 
-/// CastPtr represents the relationship between a snake case type (like rustls_client_config)
-/// and the corresponding Rust type (like ClientConfig). For each matched pair of types, there
-/// should be an `impl CastPtr for foo_bar { RustTy = FooBar }`.
+/// Used to mark that pointer to a [`Castable`]'s underlying `Castable::RustType` is provided
+/// to C code as a pointer to a `Box<Castable::RustType>`.
+pub(crate) struct OwnershipBox;
+
+/// Used to mark that a pointer to a [`Castable`]'s underlying `Castable::RustType` is provided
+/// to C code as a pointer to an `Arc<Castable::RustType>`.
+pub(crate) struct OwnershipArc;
+
+/// Used to mark that a pointer to a [`Castable`]'s underlying `Castable::RustType` is provided
+/// to C code as a pointer to a reference, `&Castable::RustType`.
+pub(crate) struct OwnershipRef;
+
+/// A trait for marking the type of a pointer to a [`Castable`]'s underlying `Castable::RustType`
+/// that is provided to C code, either a [`OwnershipBox`] when it is a pointer to a `Box<_>`,
+/// a [`OwnershipArc`] when it is a pointer to an `Arc<_>`, or a [`OwnershipRef`] when it is a
+/// pointer to a `&_`.
+#[allow(dead_code)] // This trait is only used as a marker.
+trait OwnershipMarker {}
+
+impl OwnershipMarker for OwnershipBox {}
+
+impl OwnershipMarker for OwnershipArc {}
+
+impl OwnershipMarker for OwnershipRef {}
+
+/// `Castable` represents the relationship between a snake case type (like [`client::rustls_client_config`])
+/// and the corresponding Rust type (like [`rustls::ClientConfig`]), specified as the associated type
+/// `RustType`. Each `Castable` also has an associated type `Ownership` specifying one of the
+/// [`OwnershipMarker`] types, [`OwnershipBox`], [`OwnershipArc`] or [`OwnershipRef`].
 ///
-/// This allows us to avoid using `as` in most places, and ensure that when we cast, we're
-/// preserving const-ness, and casting between the correct types.
-/// Implementing this is required in order to use `try_ref_from_ptr!` or
-/// `try_mut_from_ptr!`.
-pub(crate) trait CastPtr {
-    type RustType;
+/// An implementation of `Castable` that uses [`OwnershipBox`] indicates that when we give C code
+/// a pointer to the relevant `RustType` `T`, that it is actually a `Box<T>`. An
+/// implementation of `Castable` that uses [`OwnershipArc`] means that when we give C code a
+/// pointer to the relevant type, that it is actually an `Arc<T>`. Lastly an implementation of
+/// `Castable` that uses [`OwnershipRef`] means that when we give C code a pointer to the relevant
+/// type, that it is actually a `&T`.
+///
+/// By using an associated type on `Castable` to communicate this we can use the type system to
+/// guarantee that a single type can't implement `Castable` for more than one [`OwnershipMarker`],
+/// since this would be a conflicting trait implementation and rejected by the compiler.
+///
+/// This trait allows us to avoid using `as` in most places, and ensures that when we cast, we're
+/// preserving const-ness, and casting between the correct types. Implementing this is required in
+/// order to use `try_ref_from_ptr!` or `try_mut_from_ptr!` and several other helpful cast-related
+/// conversion helpers.
+pub(crate) trait Castable {
+    /// Indicates whether to use `Box` or `Arc` when giving a pointer to C code for the underlying
+    /// `RustType`.
+    type Ownership: OwnershipMarker;
 
-    fn cast_mut_ptr(ptr: *mut Self) -> *mut Self::RustType {
-        ptr as *mut _
-    }
+    /// The underlying Rust type that we are casting to and from.
+    type RustType;
 }
 
-/// CastConstPtr represents a subset of CastPtr, for when we can only treat
-/// something as a const (for instance when dealing with Arc).
-pub(crate) trait CastConstPtr {
-    type RustType;
-
-    fn cast_const_ptr(ptr: *const Self) -> *const Self::RustType {
-        ptr as *const _
-    }
-}
-
-/// Anything that qualifies for CastPtr also automatically qualifies for
-/// CastConstPtr. Splitting out CastPtr vs CastConstPtr allows us to ensure
-/// that Arcs are never cast to a mutable pointer.
-impl<T, R> CastConstPtr for T
+/// Convert a const pointer to a [`Castable`] to a const pointer to its underlying
+/// [`Castable::RustType`].
+///
+/// This can be used regardless of the [`Castable::Ownership`] as we can make const pointers for
+/// `Box`, `Arc` and ref types.
+pub(crate) fn cast_const_ptr<C>(ptr: *const C) -> *const C::RustType
 where
-    T: CastPtr<RustType = R>,
+    C: Castable,
 {
-    type RustType = R;
+    ptr as *const _
 }
 
-// An implementation of BoxCastPtr means that when we give C code a pointer to the relevant type,
-// it is actually a Box.
-pub(crate) trait BoxCastPtr: CastPtr + Sized {
-    fn to_box(ptr: *mut Self) -> Option<Box<Self::RustType>> {
-        if ptr.is_null() {
-            return None;
-        }
-        let rs_typed = Self::cast_mut_ptr(ptr);
-        unsafe { Some(Box::from_raw(rs_typed)) }
-    }
+/// Convert a [`Castable`]'s underlying [`Castable::RustType`] to a constant pointer
+/// to an `Arc` over the rust type. Can only be used when the `Castable` has specified a cast type
+/// equal to [`OwnershipArc`].
+pub(crate) fn to_arc_const_ptr<C>(src: C::RustType) -> *const C
+where
+    C: Castable<Ownership = OwnershipArc>,
+{
+    Arc::into_raw(Arc::new(src)) as *const _
+}
 
-    fn to_mut_ptr(src: Self::RustType) -> *mut Self {
-        Box::into_raw(Box::new(src)) as *mut _
+/// Given a const pointer to a [`Castable`] representing an `Arc`, clone the `Arc` and return
+/// the corresponding Rust type.
+///
+/// The caller still owns its copy of the `Arc`. In other words, the reference count of the
+/// `Arc` will be incremented by 1 by the end of this function.
+///
+/// To achieve that, we need to `mem::forget` the `Arc` we get back from `into_raw`, because
+/// `into_raw` _does_ take back ownership. If we called `into_raw` without `mem::forget`, at the
+/// end of the function that Arc would be dropped and the reference count would be decremented,
+/// potentially to 0, causing memory to be freed.
+///
+/// Does nothing, returning `None`, when passed a `NULL` pointer. Can only be used when the
+/// `Castable` has specified a cast type equal to [`OwnershipArc`].
+///
+/// ## Unsafety:
+///
+/// If non-null, `ptr` must be a pointer that resulted from previously calling `Arc::into_raw`,
+/// e.g. from using [`to_arc_const_ptr`].
+pub(crate) fn clone_arc<C>(ptr: *const C) -> Option<Arc<C::RustType>>
+where
+    C: Castable<Ownership = OwnershipArc>,
+{
+    if ptr.is_null() {
+        return None;
     }
+    let rs_typed = cast_const_ptr::<C>(ptr);
+    let r = unsafe { Arc::from_raw(rs_typed) };
+    let val = Arc::clone(&r);
+    mem::forget(r);
+    Some(val)
+}
 
-    fn set_mut_ptr(dst: *mut *mut Self, src: Self::RustType) {
-        unsafe {
-            *dst = Self::to_mut_ptr(src);
-        }
+/// Convert a mutable pointer to a [`Castable`] to an optional `Box` over the underlying rust type.
+///
+/// Does nothing, returning `None`, when passed `NULL`. Can only be used when the `Castable` has
+/// specified a cast type equal to [`OwnershipBox`].
+///
+/// ## Unsafety:
+///
+/// If non-null, `ptr` must be a pointer that resulted from previously calling `Box::into_raw`,
+/// e.g. from using [`to_boxed_mut_ptr`].
+pub(crate) fn to_box<C>(ptr: *mut C) -> Option<Box<C::RustType>>
+where
+    C: Castable<Ownership = OwnershipBox>,
+{
+    if ptr.is_null() {
+        return None;
+    }
+    let rs_typed = cast_mut_ptr(ptr);
+    unsafe { Some(Box::from_raw(rs_typed)) }
+}
+
+/// Free a constant pointer to a [`Castable`]'s underlying [`Castable::RustType`] by
+/// reconstituting an `Arc` from the raw pointer and dropping it.
+///
+/// For types represented with an `Arc` on the Rust side, we offer a `_free()`
+/// method to the C side that decrements the refcount and ultimately drops
+/// the `Arc` if the refcount reaches 0. By contrast with `to_arc`, we call
+/// `Arc::from_raw` on the input pointer, but we _don't_ clone it, because we
+/// want the refcount to be lower by one when we reach the end of the function.
+///
+/// Does nothing, returning `None`, when passed `NULL`. Can only be used when the `Castable` has
+/// specified a cast type equal to [`OwnershipArc`].
+pub(crate) fn free_arc<C>(ptr: *const C)
+where
+    C: Castable<Ownership = OwnershipArc>,
+{
+    if ptr.is_null() {
+        return;
+    }
+    let rs_typed = cast_const_ptr(ptr);
+    drop(unsafe { Arc::from_raw(rs_typed) });
+}
+
+/// Convert a mutable pointer to a [`Castable`] to an optional `Box` over the underlying
+/// [`Castable::RustType`], and immediately let it fall out of scope to be freed.
+///
+/// Can only be used when the `Castable` has specified a cast type equal to [`OwnershipBox`].
+///
+/// ## Unsafety:
+///
+/// If non-null, `ptr` must be a pointer that resulted from previously calling `Box::into_raw`,
+/// e.g. from using [`to_boxed_mut_ptr`].
+pub(crate) fn free_box<C>(ptr: *mut C)
+where
+    C: Castable<Ownership = OwnershipBox>,
+{
+    to_box(ptr);
+}
+
+/// Convert a mutable pointer to a [`Castable`] to a mutable pointer to its underlying
+/// [`Castable::RustType`].
+///
+/// Can only be used when the `Castable` has specified a cast source equal to `BoxCastPtrMarker`.
+pub(crate) fn cast_mut_ptr<C>(ptr: *mut C) -> *mut C::RustType
+where
+    C: Castable<Ownership = OwnershipBox>,
+{
+    ptr as *mut _
+}
+
+/// Converts a [`Castable`]'s underlying [`Castable::RustType`] to a mutable pointer
+/// to a `Box` over the rust type.
+///
+/// Can only be used when the `Castable` has specified a cast type equal to [`OwnershipBox`].
+pub(crate) fn to_boxed_mut_ptr<C>(src: C::RustType) -> *mut C
+where
+    C: Castable<Ownership = OwnershipBox>,
+{
+    Box::into_raw(Box::new(src)) as *mut _
+}
+
+/// Converts a [`Castable`]'s underlying [`Castable::RustType`] to a mutable pointer
+/// to a `Box` over the rust type and sets the `dst` out pointer to the resulting mutable `Box`
+/// pointer. See [`to_boxed_mut_ptr`] for more information.
+///
+/// ## Unsafety:
+///
+/// `dst` must not be `NULL`.
+pub(crate) fn set_boxed_mut_ptr<C>(dst: *mut *mut C, src: C::RustType)
+where
+    C: Castable<Ownership = OwnershipBox>,
+{
+    unsafe {
+        *dst = to_boxed_mut_ptr(src);
     }
 }
 
-// An implementation of ArcCastPtr means that when we give C code a pointer to the relevant type,
-// it is actually a Arc.
-pub(crate) trait ArcCastPtr: CastConstPtr + Sized {
-    /// Sometimes we create an Arc, then call `into_raw` and return the resulting raw pointer
-    /// to C. C can then call rustls_server_session_new multiple times using that
-    /// same raw pointer. On each call, we need to reconstruct the Arc. But once we reconstruct the Arc,
-    /// its reference count will be decremented on drop. We need to reference count to stay at 1,
-    /// because the C code is holding a copy. This function turns the raw pointer back into an Arc,
-    /// clones it to increment the reference count (which will make it 2 in this particular case), and
-    /// mem::forgets the clone. The mem::forget prevents the reference count from being decremented when
-    /// we exit this function, so it will stay at 2 as long as we are in Rust code. Once the caller
-    /// drops its Arc, the reference count will go back down to 1, indicating the C code's copy.
-    ///
-    /// Does nothing when passed null.
-    ///
-    /// Unsafety:
-    ///
-    /// If non-null, ptr must be a pointer that resulted from previously calling `Arc::into_raw`.
-    fn to_arc(ptr: *const Self) -> Option<Arc<Self::RustType>> {
-        if ptr.is_null() {
-            return None;
-        }
-        let rs_typed = Self::cast_const_ptr(ptr);
-        let r = unsafe { Arc::from_raw(rs_typed) };
-        let val = Arc::clone(&r);
-        mem::forget(r);
-        Some(val)
-    }
-
-    /// For types represented with an Arc on the Rust side, we offer a _free()
-    /// method to the C side that decrements the refcount and ultimately drops
-    /// the Arc if the refcount reaches 0. By contrast with to_arc, we call
-    /// Arc::from_raw on the input pointer, but we _don't_ clone it, because we
-    /// want the refcount to be lower by one when we reach the end of the function.
-    ///
-    /// Does nothing when passed null.
-    fn free(ptr: *const Self) {
-        if ptr.is_null() {
-            return;
-        }
-        let rs_typed = Self::cast_const_ptr(ptr);
-        drop(unsafe { Arc::from_raw(rs_typed) });
-    }
-
-    fn to_const_ptr(src: Self::RustType) -> *const Self {
-        Arc::into_raw(Arc::new(src)) as *const _
+/// Converts a [`Castable`]'s underlying [`Castable::RustType`] to a const pointer
+/// to an `Arc` over the rust type and sets the `dst` out pointer to the resulting const `Arc`
+/// pointer. See [`to_arc_const_ptr`] for more information.
+///
+/// ## Unsafety:
+///
+/// `dst` must not be `NULL`.
+pub(crate) fn set_arc_mut_ptr<C>(dst: *mut *const C, src: C::RustType)
+where
+    C: Castable<Ownership = OwnershipArc>,
+{
+    unsafe {
+        *dst = to_arc_const_ptr(src);
     }
 }
 
-#[doc(hidden)]
-#[macro_export]
+/// Converts a mutable pointer to a [`Castable`] to an optional ref to the underlying
+/// [`Castable::RustType`]. See [`cast_mut_ptr`] for more information.
+///
+/// Does nothing, returning `None`, when passed `NULL`. Can only be used when the `Castable` has
+/// specified a cast type equal to [`OwnershipBox`].
+pub(crate) fn try_from_mut<'a, C>(from: *mut C) -> Option<&'a mut C::RustType>
+where
+    C: Castable<Ownership = OwnershipBox>,
+{
+    unsafe { cast_mut_ptr(from).as_mut() }
+}
+
+/// If the provided pointer to a [`Castable`] is non-null, convert it to a mutable reference using
+/// [`try_from_mut`]. Otherwise, return [`rustls_result::NullParameter`], or an appropriate default
+/// (`false`, `0`, `NULL`) based on the context. See [`try_from_mut`] for more information.
+macro_rules! try_mut_from_ptr {
+    ( $var:ident ) => {
+        match $crate::try_from_mut($var) {
+            Some(c) => c,
+            None => return $crate::panic::NullParameterOrDefault::value(),
+        }
+    };
+}
+
+pub(crate) use try_mut_from_ptr;
+
+/// Converts a const pointer to a [`Castable`] to an optional ref to the underlying
+/// [`Castable::RustType`]. See [`cast_const_ptr`] for more information.
+///
+/// Does nothing, returning `None` when passed `NULL`. Can be used with `Castable`'s that
+/// specify a cast type of [`OwnershipArc`] as well as `Castable`'s that specify
+/// a cast type of [`OwnershipBox`].
+pub(crate) fn try_from<'a, C, O>(from: *const C) -> Option<&'a C::RustType>
+where
+    C: Castable<Ownership = O>,
+{
+    unsafe { cast_const_ptr(from).as_ref() }
+}
+
+/// If the provided pointer to a [`Castable`] is non-null, convert it to a reference using
+/// [`try_from`]. Otherwise, return [`rustls_result::NullParameter`], or an appropriate default
+/// (`false`, `0`, `NULL`) based on the context;
+///
+/// See [`try_from`] for more information.
+macro_rules! try_ref_from_ptr {
+    ( $var:ident ) => {
+        match $crate::try_from($var) {
+            Some(c) => c,
+            None => return $crate::panic::NullParameterOrDefault::value(),
+        }
+    };
+}
+
+pub(crate) use try_ref_from_ptr;
+
+/// If the provided pointer to a [`Castable`] is non-null, convert it to a reference to an `Arc` over
+/// the underlying rust type using [`try_arc_from`]. Otherwise, return
+/// [`rustls_result::NullParameter`], or an appropriate default (`false`, `0`, `NULL`) based on the
+/// context. See [`try_arc_from`] for more information.
+macro_rules! try_clone_arc {
+    ( $var:ident ) => {
+        match $crate::clone_arc($var) {
+            Some(c) => c,
+            None => return $crate::panic::NullParameterOrDefault::value(),
+        }
+    };
+}
+
+pub(crate) use try_clone_arc;
+
+/// Convert a mutable pointer to a [`Castable`] to an optional `Box` over the underlying
+/// [`Castable::RustType`].
+///
+/// Does nothing, returning `None`, when passed `NULL`. Can only be used with `Castable`'s that
+/// specify a cast type of [`OwnershipBox`].
+pub(crate) fn try_box_from<C>(from: *mut C) -> Option<Box<C::RustType>>
+where
+    C: Castable<Ownership = OwnershipBox>,
+{
+    to_box(from)
+}
+
+/// If the provided pointer to a [`Castable`] is non-null, convert it to a reference to a `Box`
+/// over the underlying rust type using [`try_box_from`]. Otherwise, return [`rustls_result::NullParameter`],
+/// or an appropriate default (`false`, `0`, `NULL`) based on the context. See [`try_box_from`] for
+/// more information.
+macro_rules! try_box_from_ptr {
+    ( $var:ident ) => {
+        match $crate::try_box_from($var) {
+            Some(c) => c,
+            None => return $crate::panic::NullParameterOrDefault::value(),
+        }
+    };
+}
+
+pub(crate) use try_box_from_ptr;
+
 macro_rules! try_slice {
     ( $ptr:expr, $count:expr ) => {
         if $ptr.is_null() {
@@ -409,102 +621,8 @@ macro_rules! try_slice {
     };
 }
 
-#[doc(hidden)]
-#[macro_export]
-macro_rules! try_mut_slice {
-    ( $ptr:expr, $count:expr ) => {
-        if $ptr.is_null() {
-            return $crate::panic::NullParameterOrDefault::value();
-        } else {
-            unsafe { slice::from_raw_parts_mut($ptr, $count as usize) }
-        }
-    };
-}
+pub(crate) use try_slice;
 
-/// Turn a raw const pointer into a reference. This is a generic function
-/// rather than part of the CastPtr trait because (a) const pointers can't act
-/// as "self" for trait methods, and (b) we want to rely on type inference
-/// against T (the cast-to type) rather than across F (the from type).
-pub(crate) fn try_from<'a, F, T>(from: *const F) -> Option<&'a T>
-where
-    F: CastConstPtr<RustType = T>,
-{
-    unsafe { F::cast_const_ptr(from).as_ref() }
-}
-
-/// Turn a raw mut pointer into a mutable reference.
-pub(crate) fn try_from_mut<'a, F, T>(from: *mut F) -> Option<&'a mut T>
-where
-    F: CastPtr<RustType = T>,
-{
-    unsafe { F::cast_mut_ptr(from).as_mut() }
-}
-
-pub(crate) fn try_box_from<F, T>(from: *mut F) -> Option<Box<T>>
-where
-    F: BoxCastPtr<RustType = T>,
-{
-    F::to_box(from)
-}
-
-pub(crate) fn try_arc_from<F, T>(from: *const F) -> Option<Arc<T>>
-where
-    F: ArcCastPtr<RustType = T>,
-{
-    F::to_arc(from)
-}
-
-/// If the provided pointer is non-null, convert it to a reference.
-/// Otherwise, return NullParameter, or an appropriate default (false, 0, NULL)
-/// based on the context;
-/// Example:
-///   let config: &mut ClientConfig = try_ref_from_ptr!(builder);
-#[doc(hidden)]
-#[macro_export]
-macro_rules! try_ref_from_ptr {
-    ( $var:ident ) => {
-        match $crate::try_from($var) {
-            Some(c) => c,
-            None => return $crate::panic::NullParameterOrDefault::value(),
-        }
-    };
-}
-
-#[doc(hidden)]
-#[macro_export]
-macro_rules! try_mut_from_ptr {
-    ( $var:ident ) => {
-        match $crate::try_from_mut($var) {
-            Some(c) => c,
-            None => return $crate::panic::NullParameterOrDefault::value(),
-        }
-    };
-}
-
-#[doc(hidden)]
-#[macro_export]
-macro_rules! try_box_from_ptr {
-    ( $var:ident ) => {
-        match $crate::try_box_from($var) {
-            Some(c) => c,
-            None => return $crate::panic::NullParameterOrDefault::value(),
-        }
-    };
-}
-
-#[doc(hidden)]
-#[macro_export]
-macro_rules! try_arc_from_ptr {
-    ( $var:ident ) => {
-        match $crate::try_arc_from($var) {
-            Some(c) => c,
-            None => return $crate::panic::NullParameterOrDefault::value(),
-        }
-    };
-}
-
-#[doc(hidden)]
-#[macro_export]
 macro_rules! try_callback {
     ( $var:ident ) => {
         match $var {
@@ -513,6 +631,22 @@ macro_rules! try_callback {
         }
     };
 }
+
+pub(crate) use try_callback;
+
+macro_rules! try_take {
+    ( $var:ident ) => {
+        match $var.take() {
+            None => {
+                return $crate::rustls_result::AlreadyUsed;
+            }
+            Some(x) => x,
+        }
+    };
+}
+
+pub(crate) use try_take;
+
 /// Returns a static string containing the rustls-ffi version as well as the
 /// rustls version. The string is alive for the lifetime of the program and does
 /// not need to be freed.
