@@ -1,12 +1,9 @@
 use libc::{c_char, size_t};
 use std::fmt;
 use std::marker::PhantomData;
+use std::ptr::null;
 use std::slice;
 use std::str;
-use std::{
-    convert::{TryFrom, TryInto},
-    ptr::null,
-};
 
 /// A read-only view on a Rust byte slice.
 ///
@@ -78,11 +75,9 @@ pub struct rustls_slice_slice_bytes<'a> {
 /// returns 0.
 #[no_mangle]
 pub extern "C" fn rustls_slice_slice_bytes_len(input: *const rustls_slice_slice_bytes) -> size_t {
-    unsafe {
-        match input.as_ref() {
-            Some(c) => c.inner.len(),
-            None => 0,
-        }
+    match unsafe { input.as_ref() } {
+        Some(c) => c.inner.len(),
+        None => 0,
     }
 }
 
@@ -94,8 +89,8 @@ pub extern "C" fn rustls_slice_slice_bytes_get(
     input: *const rustls_slice_slice_bytes,
     n: size_t,
 ) -> rustls_slice_bytes {
-    let input: &rustls_slice_slice_bytes = unsafe {
-        match input.as_ref() {
+    let input: &rustls_slice_slice_bytes = {
+        match unsafe { input.as_ref() } {
             Some(c) => c,
             None => {
                 return rustls_slice_bytes {
@@ -251,7 +246,6 @@ fn test_rustls_str() {
 #[cfg(test)]
 mod tests {
     use crate::rslice::*;
-    use std::convert::TryInto;
 
     #[test]
     fn test_rustls_str_debug() {
