@@ -22,6 +22,7 @@ enum rustls_result {
   RUSTLS_RESULT_PLAINTEXT_EMPTY = 7011,
   RUSTLS_RESULT_ACCEPTOR_NOT_READY = 7012,
   RUSTLS_RESULT_ALREADY_USED = 7013,
+  RUSTLS_RESULT_CERTIFICATE_REVOCATION_LIST_PARSE_ERROR = 7014,
   RUSTLS_RESULT_NO_CERTIFICATES_PRESENTED = 7101,
   RUSTLS_RESULT_DECRYPT_ERROR = 7102,
   RUSTLS_RESULT_FAILED_TO_GET_CURRENT_TIME = 7103,
@@ -104,11 +105,18 @@ enum rustls_result {
   RUSTLS_RESULT_ALERT_CERTIFICATE_REQUIRED = 7232,
   RUSTLS_RESULT_ALERT_NO_APPLICATION_PROTOCOL = 7233,
   RUSTLS_RESULT_ALERT_UNKNOWN = 7234,
-  RUSTLS_RESULT_CERT_SCT_MALFORMED = 7319,
-  RUSTLS_RESULT_CERT_SCT_INVALID_SIGNATURE = 7320,
-  RUSTLS_RESULT_CERT_SCT_TIMESTAMP_IN_FUTURE = 7321,
-  RUSTLS_RESULT_CERT_SCT_UNSUPPORTED_VERSION = 7322,
-  RUSTLS_RESULT_CERT_SCT_UNKNOWN_LOG = 7323,
+  RUSTLS_RESULT_CERT_REVOCATION_LIST_BAD_SIGNATURE = 7400,
+  RUSTLS_RESULT_CERT_REVOCATION_LIST_INVALID_CRL_NUMBER = 7401,
+  RUSTLS_RESULT_CERT_REVOCATION_LIST_INVALID_REVOKED_CERT_SERIAL_NUMBER = 7402,
+  RUSTLS_RESULT_CERT_REVOCATION_LIST_ISSUER_INVALID_FOR_CRL = 7403,
+  RUSTLS_RESULT_CERT_REVOCATION_LIST_OTHER_ERROR = 7404,
+  RUSTLS_RESULT_CERT_REVOCATION_LIST_PARSE_ERROR = 7405,
+  RUSTLS_RESULT_CERT_REVOCATION_LIST_UNSUPPORTED_CRL_VERSION = 7406,
+  RUSTLS_RESULT_CERT_REVOCATION_LIST_UNSUPPORTED_CRITICAL_EXTENSION = 7407,
+  RUSTLS_RESULT_CERT_REVOCATION_LIST_UNSUPPORTED_DELTA_CRL = 7408,
+  RUSTLS_RESULT_CERT_REVOCATION_LIST_UNSUPPORTED_INDIRECT_CRL = 7409,
+  RUSTLS_RESULT_CERT_REVOCATION_LIST_UNSUPPORTED_REVOCATION_REASON = 7410,
+  RUSTLS_RESULT_CLIENT_CERT_VERIFIER_BUILDER_NO_ROOT_ANCHORS = 7500,
 };
 typedef uint32_t rustls_result;
 
@@ -160,8 +168,8 @@ typedef struct rustls_acceptor rustls_acceptor;
 
 /**
  * An X.509 certificate, as used in rustls.
- * Corresponds to `Certificate` in the Rust API.
- * <https://docs.rs/rustls/latest/rustls/struct.Certificate.html>
+ * Corresponds to `CertificateDer` in the Rust pki-types API.
+ * <https://docs.rs/rustls-pki-types/latest/rustls_pki_types/struct.CertificateDer.html>
  */
 typedef struct rustls_certificate rustls_certificate;
 
@@ -174,24 +182,11 @@ typedef struct rustls_certificate rustls_certificate;
 typedef struct rustls_certified_key rustls_certified_key;
 
 /**
- * A verifier of client certificates that requires all certificates to be
- * trusted based on a given `rustls_root_cert_store`. Usable in building server
- * configurations. Connections without such a client certificate will not
- * be accepted.
+ * A built client certificate verifier that can be provided to a `rustls_server_config_builder`
+ * with `rustls_server_config_builder_set_client_verifier`.
  */
 typedef struct rustls_client_cert_verifier rustls_client_cert_verifier;
 
-/**
- * Alternative to `rustls_client_cert_verifier` that allows connections
- * with or without a client certificate. If the client offers a certificate,
- * it will be verified (and rejected if it is not valid). If the client
- * does not offer a certificate, the connection will succeed.
- *
- * The application can retrieve the certificate, if any, with
- * rustls_connection_get_peer_certificate.
- */
-typedef struct rustls_client_cert_verifier_optional rustls_client_cert_verifier_optional;
-
 /**
  * A client config that is done being constructed and is now read-only.
  * Under the hood, this object corresponds to an `Arc<ClientConfig>`.
@@ -226,6 +221,22 @@ typedef struct rustls_iovec rustls_iovec;
  */
 typedef struct rustls_root_cert_store rustls_root_cert_store;
 
+/**
+ * A `rustls_root_cert_store` being constructed.
+ *
+ * A builder can be modified by adding trust anchor root certificates with
+ * `rustls_root_cert_store_builder_add_pem`. Once you're done adding root certificates,
+ * call `rustls_root_cert_store_builder_build` to turn it into a `rustls_root_cert_store`.
+ * This object is not safe for concurrent mutation.
+ */
+typedef struct rustls_root_cert_store_builder rustls_root_cert_store_builder;
+
+/**
+ * A built server certificate verifier that can be provided to a `rustls_client_config_builder`
+ * with `rustls_client_config_builder_set_server_verifier`.
+ */
+typedef struct rustls_server_cert_verifier rustls_server_cert_verifier;
+
 /**
  * A server config that is done being constructed and is now read-only.
  * Under the hood, this object corresponds to an `Arc<ServerConfig>`.
@@ -284,6 +295,30 @@ typedef struct rustls_slice_str rustls_slice_str;
  */
 typedef struct rustls_supported_ciphersuite rustls_supported_ciphersuite;
 
+/**
+ * A client certificate verifier being constructed. A builder can be modified by,
+ * e.g. `rustls_web_pki_client_cert_verifier_builder_add_crl`. Once you're
+ * done configuring settings, call `rustls_web_pki_client_cert_verifier_builder_build`
+ * to turn it into a `rustls_client_cert_verifier`. This object is not safe
+ * for concurrent mutation.
+ *
+ * See <https://docs.rs/rustls/latest/rustls/server/struct.ClientCertVerifierBuilder.html>
+ * for more information.
+ */
+typedef struct rustls_web_pki_client_cert_verifier_builder rustls_web_pki_client_cert_verifier_builder;
+
+/**
+ * A server certificate verifier being constructed. A builder can be modified by,
+ * e.g. `rustls_web_pki_server_cert_verifier_builder_add_crl`. Once you're
+ * done configuring settings, call `rustls_web_pki_server_cert_verifier_builder_build`
+ * to turn it into a `rustls_server_cert_verifier`. This object is not safe
+ * for concurrent mutation.
+ *
+ * See <https://docs.rs/rustls/latest/rustls/client/struct.ServerCertVerifierBuilder.html>
+ * for more information.
+ */
+typedef struct rustls_web_pki_server_cert_verifier_builder rustls_web_pki_server_cert_verifier_builder;
+
 /**
  * A read-only view on a Rust `&str`. The contents are guaranteed to be valid
  * UTF-8. As an additional guarantee on top of Rust's normal UTF-8 guarantee,
@@ -324,7 +359,10 @@ typedef int rustls_io_result;
  * cases that should be a struct that contains, at a minimum, a file descriptor.
  * The buf and out_n pointers are borrowed and should not be retained across calls.
  */
-typedef rustls_io_result (*rustls_read_callback)(void *userdata, uint8_t *buf, size_t n, size_t *out_n);
+typedef rustls_io_result (*rustls_read_callback)(void *userdata,
+                                                 uint8_t *buf,
+                                                 size_t n,
+                                                 size_t *out_n);
 
 /**
  * A read-only view on a Rust byte slice.
@@ -364,7 +402,8 @@ typedef struct rustls_verify_server_cert_params {
   struct rustls_slice_bytes ocsp_response;
 } rustls_verify_server_cert_params;
 
-typedef uint32_t (*rustls_verify_server_cert_callback)(rustls_verify_server_cert_user_data userdata, const struct rustls_verify_server_cert_params *params);
+typedef uint32_t (*rustls_verify_server_cert_callback)(rustls_verify_server_cert_user_data userdata,
+                                                       const struct rustls_verify_server_cert_params *params);
 
 typedef size_t rustls_log_level;
 
@@ -389,7 +428,10 @@ typedef void (*rustls_log_callback)(void *userdata, const struct rustls_log_para
  * cases that should be a struct that contains, at a minimum, a file descriptor.
  * The buf and out_n pointers are borrowed and should not be retained across calls.
  */
-typedef rustls_io_result (*rustls_write_callback)(void *userdata, const uint8_t *buf, size_t n, size_t *out_n);
+typedef rustls_io_result (*rustls_write_callback)(void *userdata,
+                                                  const uint8_t *buf,
+                                                  size_t n,
+                                                  size_t *out_n);
 
 /**
  * A callback for rustls_connection_write_tls_vectored.
@@ -405,7 +447,10 @@ typedef rustls_io_result (*rustls_write_callback)(void *userdata, const uint8_t
  * cases that should be a struct that contains, at a minimum, a file descriptor.
  * The buf and out_n pointers are borrowed and should not be retained across calls.
  */
-typedef rustls_io_result (*rustls_write_vectored_callback)(void *userdata, const struct rustls_iovec *iov, size_t count, size_t *out_n);
+typedef rustls_io_result (*rustls_write_vectored_callback)(void *userdata,
+                                                           const struct rustls_iovec *iov,
+                                                           size_t count,
+                                                           size_t *out_n);
 
 /**
  * Any context information the callback will receive when invoked.
@@ -472,7 +517,8 @@ typedef struct rustls_client_hello {
  * EXPERIMENTAL: this feature of rustls-ffi is likely to change in the future, as
  * the rustls library is re-evaluating their current approach to client hello handling.
  */
-typedef const struct rustls_certified_key *(*rustls_client_hello_callback)(rustls_client_hello_userdata userdata, const struct rustls_client_hello *hello);
+typedef const struct rustls_certified_key *(*rustls_client_hello_callback)(rustls_client_hello_userdata userdata,
+                                                                           const struct rustls_client_hello *hello);
 
 /**
  * Any context information the callback will receive when invoked.
@@ -506,7 +552,12 @@ typedef void *rustls_session_store_userdata;
  * NOTE: callbacks used in several sessions via a common config
  * must be implemented thread-safe.
  */
-typedef uint32_t (*rustls_session_store_get_callback)(rustls_session_store_userdata userdata, const struct rustls_slice_bytes *key, int remove_after, uint8_t *buf, size_t count, size_t *out_n);
+typedef uint32_t (*rustls_session_store_get_callback)(rustls_session_store_userdata userdata,
+                                                      const struct rustls_slice_bytes *key,
+                                                      int remove_after,
+                                                      uint8_t *buf,
+                                                      size_t count,
+                                                      size_t *out_n);
 
 /**
  * Prototype of a callback that can be installed by the application at the
@@ -523,7 +574,9 @@ typedef uint32_t (*rustls_session_store_get_callback)(rustls_session_store_userd
  * NOTE: callbacks used in several sessions via a common config
  * must be implemented thread-safe.
  */
-typedef uint32_t (*rustls_session_store_put_callback)(rustls_session_store_userdata userdata, const struct rustls_slice_bytes *key, const struct rustls_slice_bytes *val);
+typedef uint32_t (*rustls_session_store_put_callback)(rustls_session_store_userdata userdata,
+                                                      const struct rustls_slice_bytes *key,
+                                                      const struct rustls_slice_bytes *val);
 
 extern const struct rustls_supported_ciphersuite *RUSTLS_ALL_CIPHER_SUITES[9];
 
@@ -898,15 +951,19 @@ rustls_result rustls_certified_key_clone_with_ocsp(const struct rustls_certified
 void rustls_certified_key_free(const struct rustls_certified_key *key);
 
 /**
- * Create a rustls_root_cert_store. Caller owns the memory and must
- * eventually call rustls_root_cert_store_free. The store starts out empty.
- * Caller must add root certificates with rustls_root_cert_store_add_pem.
- * <https://docs.rs/rustls/latest/rustls/struct.RootCertStore.html#method.empty>
+ * Create a `rustls_root_cert_store_builder`.
+ *
+ * Caller owns the memory and may free it with `rustls_root_cert_store_free`, regardless of
+ * whether `rustls_root_cert_store_builder_build` was called.
+ *
+ * If you wish to abandon the builder without calling `rustls_root_cert_store_builder_build`,
+ * it must be freed with `rustls_root_cert_store_builder_free`.
  */
-struct rustls_root_cert_store *rustls_root_cert_store_new(void);
+struct rustls_root_cert_store_builder *rustls_root_cert_store_builder_new(void);
 
 /**
- * Add one or more certificates to the root cert store using PEM encoded data.
+ * Add one or more certificates to the root cert store builder using PEM
+ * encoded data.
  *
  * When `strict` is true an error will return a `CertificateParseError`
  * result. So will an attempt to parse data that has zero certificates.
@@ -915,56 +972,239 @@ struct rustls_root_cert_store *rustls_root_cert_store_new(void);
  * This may be useful on systems that have syntactically invalid root
  * certificates.
  */
-rustls_result rustls_root_cert_store_add_pem(struct rustls_root_cert_store *store,
-                                             const uint8_t *pem,
-                                             size_t pem_len,
-                                             bool strict);
+rustls_result rustls_root_cert_store_builder_add_pem(struct rustls_root_cert_store_builder *builder,
+                                                     const uint8_t *pem,
+                                                     size_t pem_len,
+                                                     bool strict);
+
+/**
+ * Add one or more certificates to the root cert store builder using PEM
+ * encoded data read from the named file.
+ *
+ * When `strict` is true an error will return a `CertificateParseError`
+ * result. So will an attempt to parse data that has zero certificates.
+ *
+ * When `strict` is false, unparseable root certificates will be ignored.
+ * This may be useful on systems that have syntactically invalid root
+ * certificates.
+ */
+rustls_result rustls_root_cert_store_builder_load_roots_from_file(struct rustls_root_cert_store_builder *builder,
+                                                                  const char *filename,
+                                                                  bool strict);
+
+/**
+ * Create a new `rustls_root_cert_store` from the builder.
+ *
+ * The builder is consumed and cannot be used again, but must still be freed.
+ *
+ * The root cert store can be used in several `rustls_web_pki_client_cert_verifier_builder_new`
+ * instances and must be freed by the application when no longer needed. See the documentation of
+ * `rustls_root_cert_store_free` for details about lifetime.
+ */
+rustls_result rustls_root_cert_store_builder_build(struct rustls_root_cert_store_builder *builder,
+                                                   const struct rustls_root_cert_store **root_cert_store_out);
+
+/**
+ * Free a `rustls_root_cert_store_builder` previously returned from
+ * `rustls_root_cert_store_builder_new`. Calling with NULL is fine. Must not be
+ * called twice with the same value.
+ */
+void rustls_root_cert_store_builder_free(struct rustls_root_cert_store_builder *builder);
 
 /**
  * Free a rustls_root_cert_store previously returned from rustls_root_cert_store_builder_build.
  * Calling with NULL is fine. Must not be called twice with the same value.
  */
-void rustls_root_cert_store_free(struct rustls_root_cert_store *store);
+void rustls_root_cert_store_free(const struct rustls_root_cert_store *store);
 
 /**
- * Create a new client certificate verifier for the root store. The verifier
- * can be used in several rustls_server_config instances. Must be freed by
- * the application when no longer needed. See the documentation of
- * rustls_client_cert_verifier_free for details about lifetime.
- * This copies the contents of the rustls_root_cert_store. It does not take
- * ownership of the pointed-to memory.
+ * Free a `rustls_client_cert_verifier` previously returned from
+ * `rustls_client_cert_verifier_builder_build`. Calling with NULL is fine. Must not be
+ * called twice with the same value.
  */
-const struct rustls_client_cert_verifier *rustls_client_cert_verifier_new(const struct rustls_root_cert_store *store);
+void rustls_client_cert_verifier_free(struct rustls_client_cert_verifier *verifier);
 
 /**
- * "Free" a verifier previously returned from
- * rustls_client_cert_verifier_new. Since rustls_client_cert_verifier is actually an
- * atomically reference-counted pointer, extant server_configs may still
- * hold an internal reference to the Rust object. However, C code must
- * consider this pointer unusable after "free"ing it.
- * Calling with NULL is fine. Must not be called twice with the same value.
- */
-void rustls_client_cert_verifier_free(const struct rustls_client_cert_verifier *verifier);
-
-/**
- * Create a new rustls_client_cert_verifier_optional for the root store. The
- * verifier can be used in several rustls_server_config instances. Must be
- * freed by the application when no longer needed. See the documentation of
- * rustls_client_cert_verifier_optional_free for details about lifetime.
- * This copies the contents of the rustls_root_cert_store. It does not take
+ * Create a `rustls_web_pki_client_cert_verifier_builder`. Caller owns the memory and may
+ * eventually call `rustls_web_pki_client_cert_verifier_builder_free` to free it, whether or
+ * not `rustls_web_pki_client_cert_verifier_builder_build` was called.
+ *
+ * Without further modification the builder will produce a client certificate verifier that
+ * will require a client present a client certificate that chains to one of the trust anchors
+ * in the provided `rustls_root_cert_store`. The root cert store must not be empty.
+ *
+ * Revocation checking will not be performed unless
+ * `rustls_web_pki_client_cert_verifier_builder_add_crl` is used to add certificate revocation
+ * lists (CRLs) to the builder. If CRLs are added, revocation checking will be performed
+ * for the entire certificate chain unless
+ * `rustls_web_pki_client_cert_verifier_only_check_end_entity_revocation` is used. Unknown
+ * revocation status for certificates considered for revocation status will be treated as
+ * an error unless `rustls_web_pki_client_cert_verifier_allow_unknown_revocation_status` is
+ * used.
+ *
+ * Unauthenticated clients will not be permitted unless
+ * `rustls_web_pki_client_cert_verifier_builder_allow_unauthenticated` is used.
+ *
+ * This copies the contents of the `rustls_root_cert_store`. It does not take
  * ownership of the pointed-to data.
  */
-const struct rustls_client_cert_verifier_optional *rustls_client_cert_verifier_optional_new(const struct rustls_root_cert_store *store);
+struct rustls_web_pki_client_cert_verifier_builder *rustls_web_pki_client_cert_verifier_builder_new(const struct rustls_root_cert_store *store);
 
 /**
- * "Free" a verifier previously returned from
- * rustls_client_cert_verifier_optional_new. Since rustls_client_cert_verifier_optional
- * is actually an atomically reference-counted pointer, extant server_configs may still
- * hold an internal reference to the Rust object. However, C code must
- * consider this pointer unusable after "free"ing it.
- * Calling with NULL is fine. Must not be called twice with the same value.
+ * Add one or more certificate revocation lists (CRLs) to the client certificate verifier
+ * builder by reading the CRL content from the provided buffer of PEM encoded content.
+ *
+ * By default revocation checking will be performed on the entire certificate chain. To only
+ * check the revocation status of the end entity certificate, use
+ * `rustls_web_pki_client_cert_verifier_only_check_end_entity_revocation`.
+ *
+ * This function returns an error if the provided buffer is not valid PEM encoded content.
  */
-void rustls_client_cert_verifier_optional_free(const struct rustls_client_cert_verifier_optional *verifier);
+rustls_result rustls_web_pki_client_cert_verifier_builder_add_crl(struct rustls_web_pki_client_cert_verifier_builder *builder,
+                                                                  const uint8_t *crl_pem,
+                                                                  size_t crl_pem_len);
+
+/**
+ * When CRLs are provided with `rustls_web_pki_client_cert_verifier_builder_add_crl`, only
+ * check the revocation status of end entity certificates, ignoring any intermediate certificates
+ * in the chain.
+ */
+rustls_result rustls_web_pki_client_cert_verifier_only_check_end_entity_revocation(struct rustls_web_pki_client_cert_verifier_builder *builder);
+
+/**
+ * When CRLs are provided with `rustls_web_pki_client_cert_verifier_builder_add_crl`, and it
+ * isn't possible to determine the revocation status of a considered certificate, do not treat
+ * it as an error condition.
+ *
+ * Overrides the default behavior where unknown revocation status is considered an error.
+ */
+rustls_result rustls_web_pki_client_cert_verifier_allow_unknown_revocation_status(struct rustls_web_pki_client_cert_verifier_builder *builder);
+
+/**
+ * Allow unauthenticated anonymous clients in addition to those that present a client
+ * certificate that chains to one of the verifier's configured trust anchors.
+ */
+rustls_result rustls_web_pki_client_cert_verifier_builder_allow_unauthenticated(struct rustls_web_pki_client_cert_verifier_builder *builder);
+
+/**
+ * Clear the list of trust anchor hint subjects.
+ *
+ * By default, the client cert verifier will use the subjects provided by the root cert
+ * store configured for client authentication. Calling this function will remove these
+ * hint subjects, indicating the client should make a free choice of which certificate
+ * to send.
+ */
+rustls_result rustls_web_pki_client_cert_verifier_clear_root_hint_subjects(struct rustls_web_pki_client_cert_verifier_builder *builder);
+
+/**
+ * Add additional distinguished names to the list of trust anchor hint subjects.
+ *
+ * By default, the client cert verifier will use the subjects provided by the root cert
+ * store configured for client authentication. Calling this function will add to these
+ * existing hint subjects. Calling this function with an empty `store` will have no
+ * effect, use `rustls_web_pki_client_cert_verifier_clear_root_hint_subjects` to clear
+ * the subject hints.
+ */
+rustls_result rustls_web_pki_client_cert_verifier_add_root_hint_subjects(struct rustls_web_pki_client_cert_verifier_builder *builder,
+                                                                         const struct rustls_root_cert_store *store);
+
+/**
+ * Create a new client certificate verifier from the builder.
+ *
+ * The builder is consumed and cannot be used again, but must still be freed.
+ *
+ * The verifier can be used in several `rustls_server_config` instances and must be
+ * freed by the application when no longer needed. See the documentation of
+ * `rustls_web_pki_client_cert_verifier_builder_free` for details about lifetime.
+ */
+rustls_result rustls_web_pki_client_cert_verifier_builder_build(struct rustls_web_pki_client_cert_verifier_builder *builder,
+                                                                struct rustls_client_cert_verifier **verifier_out);
+
+/**
+ * Free a `rustls_client_cert_verifier_builder` previously returned from
+ * `rustls_client_cert_verifier_builder_new`. Calling with NULL is fine. Must not be
+ * called twice with the same value.
+ */
+void rustls_web_pki_client_cert_verifier_builder_free(struct rustls_web_pki_client_cert_verifier_builder *builder);
+
+/**
+ * Create a `rustls_web_pki_server_cert_verifier_builder`. Caller owns the memory and may
+ * free it with `rustls_web_pki_server_cert_verifier_builder_free`, regardless of whether
+ * `rustls_web_pki_server_cert_verifier_builder_build` was called.
+ *
+ * Without further modification the builder will produce a server certificate verifier that
+ * will require a server present a certificate that chains to one of the trust anchors
+ * in the provided `rustls_root_cert_store`. The root cert store must not be empty.
+ *
+ * Revocation checking will not be performed unless
+ * `rustls_web_pki_server_cert_verifier_builder_add_crl` is used to add certificate revocation
+ * lists (CRLs) to the builder.  If CRLs are added, revocation checking will be performed
+ * for the entire certificate chain unless
+ * `rustls_web_pki_server_cert_verifier_only_check_end_entity_revocation` is used. Unknown
+ * revocation status for certificates considered for revocation status will be treated as
+ * an error unless `rustls_web_pki_server_cert_verifier_allow_unknown_revocation_status` is
+ * used.
+ *
+ * This copies the contents of the `rustls_root_cert_store`. It does not take
+ * ownership of the pointed-to data.
+ */
+struct rustls_web_pki_server_cert_verifier_builder *rustls_web_pki_server_cert_verifier_builder_new(const struct rustls_root_cert_store *store);
+
+/**
+ * Add one or more certificate revocation lists (CRLs) to the server certificate verifier
+ * builder by reading the CRL content from the provided buffer of PEM encoded content.
+ *
+ * By default revocation checking will be performed on the entire certificate chain. To only
+ * check the revocation status of the end entity certificate, use
+ * `rustls_web_pki_server_cert_verifier_only_check_end_entity_revocation`.
+ *
+ * This function returns an error if the provided buffer is not valid PEM encoded content.
+ */
+rustls_result rustls_web_pki_server_cert_verifier_builder_add_crl(struct rustls_web_pki_server_cert_verifier_builder *builder,
+                                                                  const uint8_t *crl_pem,
+                                                                  size_t crl_pem_len);
+
+/**
+ * When CRLs are provided with `rustls_web_pki_server_cert_verifier_builder_add_crl`, only
+ * check the revocation status of end entity certificates, ignoring any intermediate certificates
+ * in the chain.
+ */
+rustls_result rustls_web_pki_server_cert_verifier_only_check_end_entity_revocation(struct rustls_web_pki_server_cert_verifier_builder *builder);
+
+/**
+ * When CRLs are provided with `rustls_web_pki_server_cert_verifier_builder_add_crl`, and it
+ * isn't possible to determine the revocation status of a considered certificate, do not treat
+ * it as an error condition.
+ *
+ * Overrides the default behavior where unknown revocation status is considered an error.
+ */
+rustls_result rustls_web_pki_server_cert_verifier_allow_unknown_revocation_status(struct rustls_web_pki_server_cert_verifier_builder *builder);
+
+/**
+ * Create a new server certificate verifier from the builder.
+ *
+ * The builder is consumed and cannot be used again, but must still be freed.
+ *
+ * The verifier can be used in several `rustls_client_config` instances and must be
+ * freed by the application when no longer needed. See the documentation of
+ * `rustls_web_pki_server_cert_verifier_builder_free` for details about lifetime.
+ */
+rustls_result rustls_web_pki_server_cert_verifier_builder_build(struct rustls_web_pki_server_cert_verifier_builder *builder,
+                                                                struct rustls_server_cert_verifier **verifier_out);
+
+/**
+ * Free a `rustls_server_cert_verifier_builder` previously returned from
+ * `rustls_server_cert_verifier_builder_new`. Calling with NULL is fine. Must not be
+ * called twice with the same value.
+ */
+void rustls_web_pki_server_cert_verifier_builder_free(struct rustls_web_pki_server_cert_verifier_builder *builder);
+
+/**
+ * Free a `rustls_server_cert_verifier` previously returned from
+ * `rustls_server_cert_verifier_builder_build`. Calling with NULL is fine. Must not be
+ * called twice with the same value.
+ */
+void rustls_server_cert_verifier_free(struct rustls_server_cert_verifier *verifier);
 
 /**
  * Create a rustls_client_config_builder. Caller owns the memory and must
@@ -1032,22 +1272,12 @@ rustls_result rustls_client_config_builder_dangerous_set_certificate_verifier(st
                                                                               rustls_verify_server_cert_callback callback);
 
 /**
- * Use the trusted root certificates from the provided store.
+ * Configure the server certificate verifier.
  *
- * This replaces any trusted roots already configured with copies
- * from `roots`. This adds 1 to the refcount for `roots`. When you
- * call rustls_client_config_free or rustls_client_config_builder_free,
- * those will subtract 1 from the refcount for `roots`.
+ * This increases the reference count of `verifier` and does not take ownership.
  */
-rustls_result rustls_client_config_builder_use_roots(struct rustls_client_config_builder *config_builder,
-                                                     const struct rustls_root_cert_store *roots);
-
-/**
- * Add trusted root certificates from the named file, which should contain
- * PEM-formatted certificates.
- */
-rustls_result rustls_client_config_builder_load_roots_from_file(struct rustls_client_config_builder *config_builder,
-                                                                const char *filename);
+void rustls_client_config_builder_set_server_verifier(struct rustls_client_config_builder *builder,
+                                                      const struct rustls_server_cert_verifier *verifier);
 
 /**
  * Set the ALPN protocol list to the given protocols. `protocols` must point
@@ -1416,23 +1646,12 @@ rustls_result rustls_server_config_builder_new_custom(const struct rustls_suppor
                                                       struct rustls_server_config_builder **builder_out);
 
 /**
- * Create a rustls_server_config_builder for TLS sessions that require
- * valid client certificates. The passed rustls_client_cert_verifier may
- * be used in several builders.
- * For memory lifetime, see rustls_server_config_builder_new.
+ * Create a rustls_server_config_builder for TLS sessions that may verify client
+ * certificates. This increases the refcount of `verifier` and doesn't take ownership.
  */
 void rustls_server_config_builder_set_client_verifier(struct rustls_server_config_builder *builder,
                                                       const struct rustls_client_cert_verifier *verifier);
 
-/**
- * Create a rustls_server_config_builder for TLS sessions that accept
- * valid client certificates, but do not require them. The passed
- * rustls_client_cert_verifier_optional may be used in several builders.
- * For memory lifetime, see rustls_server_config_builder_new.
- */
-void rustls_server_config_builder_set_client_verifier_optional(struct rustls_server_config_builder *builder,
-                                                               const struct rustls_client_cert_verifier_optional *verifier);
-
 /**
  * "Free" a server_config_builder without building it into a rustls_server_config.
  * Normally builders are built into rustls_server_configs via `rustls_server_config_builder_build`
