@@ -9,14 +9,10 @@
 #else
 #include <sys/socket.h>
 #include <sys/uio.h>
-#include <netinet/in.h>
-#include <netdb.h>
-#include <arpa/inet.h>
-#include <unistd.h>
 #include <fcntl.h>
+#include <unistd.h>
 #endif
 
-#include <sys/types.h>
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
@@ -25,13 +21,16 @@
 #include "rustls.h"
 #include "common.h"
 
+/* Set by client.c's and server.c's main() */
+const char *programname;
+
 void
-print_error(const char *program_name, const char *prefix, rustls_result result)
+print_error(const char *prefix, rustls_result result)
 {
   char buf[256];
   size_t n;
   rustls_error(result, buf, sizeof(buf), &n);
-  fprintf(stderr, "%s: %s: %.*s\n", program_name, prefix, (int)n, buf);
+  LOG("%s: %.*s", prefix, (int)n, buf);
 }
 
 #ifdef _WIN32
@@ -49,28 +48,6 @@ ws_strerror(int err)
 }
 #endif
 
-/*
- * Write n bytes from buf to the provided fd (on Windows, this must be
- * stdout/stderr or a file, not a socket), retrying short writes until
- * we finish or hit an error. Assumes fd is blocking and therefore doesn't
- * handle EAGAIN. Returns 0 for success or 1 for error.
- */
-int
-write_all(int fd, const char *buf, int n)
-{
-  int m = 0;
-
-  while(n > 0) {
-    m = write(fd, buf, n);
-    if(m < 0) {
-      perror("write_all");
-      return 1;
-    }
-    n -= m;
-  }
-  return 0;
-}
-
 /*
  * Set a socket to be nonblocking.
  *
@@ -105,9 +82,8 @@ nonblock(int sockfd)
 int
 read_cb(void *userdata, unsigned char *buf, size_t len, size_t *out_n)
 {
-  ssize_t n = 0;
   struct conndata *conn = (struct conndata *)userdata;
-  n = recv(conn->fd, buf, len, 0);
+  ssize_t n = recv(conn->fd, buf, len, 0);
   if(n < 0) {
     return errno;
   }
@@ -120,10 +96,9 @@ read_cb(void *userdata, unsigned char *buf, size_t len, size_t *out_n)
 int
 write_cb(void *userdata, const unsigned char *buf, size_t len, size_t *out_n)
 {
-  ssize_t n = 0;
   struct conndata *conn = (struct conndata *)userdata;
 
-  n = send(conn->fd, buf, len, 0);
+  ssize_t n = send(conn->fd, buf, len, 0);
   if(n < 0) {
     return errno;
   }
@@ -140,21 +115,26 @@ write_tls(struct rustls_connection *rconn, struct conndata *conn, size_t *n)
   return rustls_connection_write_tls(rconn, write_cb, conn, n);
 #else
   if(getenv("VECTORED_IO")) {
-    return rustls_connection_write_tls_vectored(rconn, write_vectored_cb, conn, n);
-  } else {
+    return rustls_connection_write_tls_vectored(
+      rconn, write_vectored_cb, conn, n);
+  }
+  else {
     return rustls_connection_write_tls(rconn, write_cb, conn, n);
   }
 #endif /* _WIN32 */
 }
 
 #ifndef _WIN32
-rustls_io_result write_vectored_cb(
-    void *userdata, const struct rustls_iovec *iov, size_t count, size_t *out_n)
+rustls_io_result
+write_vectored_cb(void *userdata, const struct rustls_iovec *iov, size_t count,
+                  size_t *out_n)
 {
-  ssize_t n = 0;
   struct conndata *conn = (struct conndata *)userdata;
 
-  n = writev(conn->fd, (const struct iovec *)iov, count);
+  // safety: narrowing conversion from `size_t count` to `int` is safe because
+  // writev return -1 and sets errno to EINVAL on out of range input (<0 || >
+  // IOV_MAX).
+  ssize_t n = writev(conn->fd, (const struct iovec *)iov, (int)count);
   if(n < 0) {
     return errno;
   }
@@ -213,7 +193,7 @@ bytevec_ensure_available(struct bytevec *vec, size_t n)
 int
 copy_plaintext_to_buffer(struct conndata *conn)
 {
-  int result;
+  unsigned int result;
   size_t n;
   struct rustls_connection *rconn = conn->rconn;
 
@@ -230,7 +210,7 @@ copy_plaintext_to_buffer(struct conndata *conn)
       return DEMO_OK;
     }
     if(result != RUSTLS_RESULT_OK) {
-      print_error(conn->program_name, "Error in rustls_connection_read", result);
+      print_error("error in rustls_connection_read", result);
       return DEMO_ERROR;
     }
     if(n == 0) {
@@ -242,8 +222,6 @@ copy_plaintext_to_buffer(struct conndata *conn)
       return DEMO_ERROR;
     }
   }
-
-  return DEMO_ERROR;
 }
 
 /**
@@ -348,8 +326,62 @@ get_first_header_value(const char *headers, size_t headers_len,
 void
 log_cb(void *userdata, const struct rustls_log_params *params)
 {
-  struct conndata *conn = (struct conndata*)userdata;
+  struct conndata *conn = (struct conndata *)userdata;
   struct rustls_str level_str = rustls_log_level_str(params->level);
-  fprintf(stderr, "%s[fd %d][%.*s]: %.*s\n", conn->program_name, conn->fd,
-    (int)level_str.len, level_str.data, (int)params->message.len, params->message.data);
+  LOG("[fd %d][%.*s]: %.*s",
+      conn->fd,
+      (int)level_str.len,
+      level_str.data,
+      (int)params->message.len,
+      params->message.data);
+}
+
+enum demo_result
+read_file(const char *filename, char *buf, size_t buflen, size_t *n)
+{
+  FILE *f = fopen(filename, "r");
+  if(f == NULL) {
+    LOG("opening %s: %s", filename, strerror(errno));
+    return DEMO_ERROR;
+  }
+  *n = fread(buf, 1, buflen, f);
+  if(!feof(f)) {
+    LOG("reading %s: %s", filename, strerror(errno));
+    fclose(f);
+    return DEMO_ERROR;
+  }
+  fclose(f);
+  return DEMO_OK;
+}
+
+const struct rustls_certified_key *
+load_cert_and_key(const char *certfile, const char *keyfile)
+{
+  char certbuf[10000];
+  size_t certbuf_len;
+  char keybuf[10000];
+  size_t keybuf_len;
+
+  unsigned int result =
+    read_file(certfile, certbuf, sizeof(certbuf), &certbuf_len);
+  if(result != DEMO_OK) {
+    return NULL;
+  }
+
+  result = read_file(keyfile, keybuf, sizeof(keybuf), &keybuf_len);
+  if(result != DEMO_OK) {
+    return NULL;
+  }
+
+  const struct rustls_certified_key *certified_key;
+  result = rustls_certified_key_build((uint8_t *)certbuf,
+                                      certbuf_len,
+                                      (uint8_t *)keybuf,
+                                      keybuf_len,
+                                      &certified_key);
+  if(result != RUSTLS_RESULT_OK) {
+    print_error("parsing certificate and key", result);
+    return NULL;
+  }
+  return certified_key;
 }
